      program readrgb
c
c  read rgb file and get con-centric info
c
c  na:  number of angles (na=360)
c  ngm: number of max gates (ngm=800)
c  da:  differentail of angle (da=360/na)
c  nb1: number of gates in a bin (nb1=5)
c  nbm: number of max bins (nbm=ngm/nb1)
c  mp,np:  number of pixels of image in x and y direction
c  nsc: number of sectors
c  nas: number of rays per sector
c  ngo: nubmer of gates in a ray to be output
c  nbo: nubmer of gates in a ray to be output
c  j0: is the bottom data gate in y direction, below j0 is the color table
c  nsctm2: min gates for computing a sector mean
c  nmean2: min gates for computing a ring mean
c
      parameter (mp=800,np=800,j0=61,md=20,d2r=3.14159265358979/180.)
      parameter (na=360,nb1=5,ngm=800,nbm=ngm/nb1,da=360./na)
      parameter (nsc=8,nas=na/nsc,n23=na*2/3,ngo=300,nbo=50)

      dimension dr(ngm,na),db(nbm,na),dc(nbm,nsc),p(mp,np)
      dimension d23(nbm),rmean(nbm),sdl(nbm),sdh(nbm),std(nbm)
      dimension aring(na),smean(nbm)
      dimension ir(800),ig(800),ib(800),t(800)
      byte r(mp,np),g(mp,np),b(mp,np)
      character*80 input*70,output
      data spv/-999./

      nmean2=na*0.75
      nsctm2=nas*0.75

      open(10,file='../navy800.txt',
     +        form='formatted',status='old')
      do i=1,800
         read(10,*)ii,ir(i),ig(i),ib(i),t(i)
      enddo
      close(10)

  10  continue

      print*,'input rgb file'
      read(*,'(a)',end=99)input
      read(*,*)ipx,jpx
c     jpx=np-jpx+1
      xpc=ipx
      ypc=jpx
      ix=index(input,'.rgb')-1
      ix4=ix+4
      print*,input(1:ix4),ipx,jpx

      open(11,file=input,form='unformatted',status='old',
     +        access='direct',recl=mp*np*3,err=10)

c     output=input(1:ix)//'.240'
c     open(21,file=output,form='formatted',status='unknown')

      output=input(1:ix)//'.bin'
      open(32,file=output,form='unformatted',access='direct',
     &     recl=4*mp*np,status='replace')

      output=input(1:ix)//'.sct'
      open(22,file=output,form='formatted',status='unknown')

      output=input(1:ix)//'.ray'
      open(23,file=output,form='unformatted',status='unknown')

      read(11,rec=1)((r(i,j),g(i,j),b(i,j),i=1,mp),j=np,1,-1)
c
c extract data for each gate, save in p
c
      do i=1,mp
      do j=1,np
         ir1=r(i,j)
         if(ir1.lt.0)ir1=ir1+256
         ig1=g(i,j)
         if(ig1.lt.0)ig1=ig1+256
         ib1=b(i,j)
         if(ib1.lt.0)ib1=ib1+256

         id1=iabs(ir1-ig1)
         id2=iabs(ir1-ib1)
         id3=iabs(ib1-ig1)
         if(id1.le.md.and.id2.le.md.and.id3.le.md)then
            p(i,j)=spv
            r(i,j)=0
            g(i,j)=0
            b(i,j)=0
         else if(ir1.le.5.and.ig1.le.5.and.ib1.le.5)then
            p(i,j)=spv
            r(i,j)=0
            g(i,j)=0
            b(i,j)=0
         else if(ir1.ge.128.and.ig1.ge.128.and.ib1.ge.128)then
            p(i,j)=spv
            r(i,j)=0
            g(i,j)=0
            b(i,j)=0
         else
            p(i,j)=rgbtemp(ir,ig,ib,t,ir1,ib1,ig1)
         endif
      enddo
      enddo
      write(32,rec=1) (((p(i,j)),i=1,mp),j=1,np)
c
c  map temp back to rgb, for double checking purpose
c
      call remap(p,r,g,b)

      do j=jpx-1,jpx+1
      do i=ipx-20,ipx+20
         r(i,j)=-128
         g(i,j)=0
         b(i,j)=0
      enddo
      enddo
      do j=jpx-20,jpx+20
      do i=ipx-1,ipx+1
         r(i,j)=-128
         g(i,j)=0
         b(i,j)=0
      enddo
      enddo

      write(11,rec=1)((r(i,j),g(i,j),b(i,j),i=1,mp),j=np,1,-1)
      close(11)
c
c  get ray data, save in dr
c
      do ia=1,na
         ang=ia*da
         cosa=cos(ang*d2r)
         sina=sin(ang*d2r)
         do igt=1,ngm
            range=igt
            ix=range*cosa+0.5+xpc
            iy=range*sina+0.5+ypc
            if(ix.ge.1.and.ix.le.mp .and. iy.ge.j0.and.iy.le.np)then
               dr(igt,ia)=p(ix,iy)
            else
               dr(igt,ia)=spv
            endif
         enddo
      enddo

c
c  compute nb1 gates average (for one bin), save in db
c
      do ia=1,na
         do ibn=1,nbm
            igt1=(ibn-1)*nb1+1
            igt2=ibn*nb1
            db(ibn,ia)=0.
            isum=0
            do igt=igt1,igt2
               if(dr(igt,ia).ne.spv)then
                  db(ibn,ia)=db(ibn,ia)+dr(igt,ia)
                  isum=isum+1
               endif
            enddo
            if(isum.gt.0)then
               db(ibn,ia)=db(ibn,ia)/isum
            else
               db(ibn,ia)=spv
            endif
         enddo
      enddo
c
c  find sector mean and ring mean from db
c
      do ibn=1,nbm
         smean(ibn)=0.
         ism=0
         do is=1,nsc
            ia1=(is-1)*nas+1
            ia2=is*nas
            dc(ibn,is)=0.
            isum=0
            do ia=ia1,ia2
               if(db(ibn,ia).ne.spv)then
                  dc(ibn,is)=dc(ibn,is)+db(ibn,ia)
                  isum=isum+1
               endif
            enddo
            if(isum.ge.nsctm2)then
               dc(ibn,is)=dc(ibn,is)/isum
               smean(ibn)=smean(ibn)+dc(ibn,is)
               ism=ism+1
            else
               dc(ibn,is)=spv
            endif
         enddo
c
c  compute sector mean
c
         if(ism.gt.0)then
            smean(ibn)=smean(ibn)/ism
         else
            smean(ibn)=spv
         endif
c
c  find ring mean
c  compute standard deviation along a ring
c
         rmean(ibn)=0.
         irm=0
         do ia=1,na
            if(db(ibn,ia).ne.spv)then
               irm=irm+1
               rmean(ibn)=rmean(ibn)+db(ibn,ia)
               aring(irm)=db(ibn,ia)
            endif
         enddo
         if(irm.ge.nmean2)then
            rmean(ibn)=rmean(ibn)/irm
            std(ibn)=xstand(aring,irm)
            sdl(ibn)=rmean(ibn)-std(ibn)
            sdh(ibn)=rmean(ibn)+std(ibn)
         else
            rmean(ibn)=spv
            std(ibn)=spv
            sdl(ibn)=spv
            sdh(ibn)=spv
         endif
c
c  ascend sorting on aring
c
         if(irm.ge.n23)then
            call sort(aring,irm)
            d23(ibn)=aring(n23)
         else
            d23(ibn)=spv
         endif
      enddo
c
c  output mean data
c
      do i=1,nbm
         x=i
c        write(21,21)x,d23(i),rmean(i),sdl(i),sdh(i),std(i)
         write(22,21)x,d23(i),rmean(i),sdl(i),sdh(i),std(i),
     +                (dc(i,j),j=1,nsc),smean(i)
  21     format(f4.0,14f8.2)
      enddo
      close(21)
      close(22)
c
c  output ray data
c

c     write(23)((dr(igt,ia),igt=1,ngo),ia=1,na)
      write(23)((db(ibn,ia),ibn=1,nbo),ia=1,na)
      close(23)

  99  continue

      stop
      end

      subroutine sort(dat,npt)
c
c ascending
c
      dimension dat(npt)

      if(npt.le.1)return
      do i=1,npt
         do j=i+1,npt
            if(dat(i).gt.dat(j))then
               r1=dat(i)
               dat(i)=dat(j)
               dat(j)=r1
            endif
         enddo
      enddo
      return
      end

      function rgbtemp(ir,ig,ib,t,ir1,ib1,ig1)
c
c  find the cloest color to determine t
c
      dimension ir(800),ig(800),ib(800),t(800)

      idf=100000000
      do i=1,800
         id1=(ir(i)-ir1)**2+(ig(i)-ig1)**2+(ib(i)-ib1)**2
         if(id1.lt.idf)then
            ii=i
            idf=id1
         endif
      enddo
      rgbtemp=t(ii)
      return
      end

      function xmean(x,n)
c
c  find mean of x(n)
c
      dimension x(n)

      spv=-999.
      xmean=0.
      im=0
      do i=1,n
         if(x(i).ne.spv)then
            xmean=xmean+x(i)
            im=im+1
         endif
      enddo
      if(im.gt.0)then
         xmean=xmean/im
      else
         xmean=0.
         print*,'no mean is found',char(7)
      endif
      return
      end

      subroutine xymean(x,y,n,xm,ym)
c
c  find mean of x(n),y(n)
c
      dimension x(n),y(n)

      spv=-999.
      xm=0.
      ym=0.
      im=0
      do i=1,n
         if(x(i).ne.spv.and.y(i).ne.spv)then
            xm=xm+x(i)
            ym=ym+y(i)
            im=im+1
         endif
      enddo
      if(im.le.0)return
         xm=xm/im
         ym=ym/im
      return
      end

      function xstand(x,n)
c
c  find standard deviation of x(n)
c
      dimension x(n)

      spv=-999.
      xstand=0.
      im=0
      xm=xmean(x,n)
      do i=1,n
         if(x(i).ne.spv)then
            xstand=xstand+(x(i)-xm)**2
            im=im+1
         endif
      enddo
      if(im.gt.2)then
         xstand=sqrt(xstand/(im-1))
      else
         xstand=0.
         print*,'no standard deviation is found ',im,char(7)
      endif
      return
      end

      function xcorrel(x,y,n)
c
c  find correc
c
      dimension x(n),y(n)

      spv=-999.
      xm=0.
      ym=0.
      xx=0.
      yy=0.
      xy=0.
      im=0
      xcorrel=0.
      do i=1,n
         if(x(i).ne.spv.and.y(i).ne.spv)then
            xm=xm+x(i)
            ym=ym+y(i)
            im=im+1
         endif
      enddo
      if(im.le.0)return
      xm=xm/im
      ym=ym/im
      do i=1,n
         if(x(i).ne.spv.and.y(i).ne.spv)then
            xx=xx+(x(i)-xm)**2
            yy=yy+(y(i)-ym)**2
            xy=xy+(x(i)-xm)*(y(i)-ym)
         endif
      enddo
      xcorrel=xy/sqrt(xx*yy)
      return
      end

      subroutine regre(x,y,n,a,b)
c
c  regre y=ax+b
c
      dimension x(n),y(n)

      spv=-999.
      xm=0.
      ym=0.
      im=0
      a=0.
      b=0.
      do i=1,n
         if(x(i).ne.spv.and.y(i).ne.spv)then
            xm=xm+x(i)
            ym=ym+y(i)
            im=im+1
         endif
      enddo
      if(im.le.0)return
      xm=xm/im
      ym=ym/im
      xx=0.
      xy=0.
      do i=1,n
         if(x(i).ne.spv.and.y(i).ne.spv)then
            xx=xx+(x(i)-xm)**2
            xy=xy+(x(i)-xm)*(y(i)-ym)
         endif
      enddo
      a=xy/xx
      b=ym-a*xm
      return
      end

      subroutine remap(p,r,g,b)
c
c  remap p back to RGB
c
      parameter (m=800,n=800,nc=800)
      dimension p(m,n),ir(nc),ig(nc),ib(nc),tt(nc)
      byte r(m,n),g(m,n),b(m,n)

      spv=-999.
      open(19,file='../navy800.txt',
     +        form='formatted',status='old')
c     open(19,file='/home/chliu/amanda/data/color/navy.256',
c    +        form='formatted',status='old')
c     read(19,*)
      do i=1,nc
         read(19,*)ii,ir(i),ig(i),ib(i),tt(i)
      enddo
      close(19)

      do j=1,n
      do i=1,m
         if(p(i,j).eq.spv)then
            r(i,j)=0
            g(i,j)=0
            b(i,j)=0
         else
            do k=1,nc
               if(p(i,j).le.tt(k))then
                  ir1=ir(k)
                  if(ir1.ge.128)ir1=ir1-256
                  ig1=ig(k)
                  if(ig1.ge.128)ig1=ig1-256
                  ib1=ib(k)
                  if(ib1.ge.128)ib1=ib1-256
                  r(i,j)=ir1
                  g(i,j)=ig1
                  b(i,j)=ib1
                  goto 10
               endif
            enddo
         endif
  10     continue
      enddo
      enddo
      return
      end

